//! This shows a very basic example of running code on the LP core.
//!
//! Code on LP core increments a counter and continuously toggles LED. The
//! current value is printed by the HP core.
//!
//! Make sure to first compile the `esp-lp-hal/examples/blinky.rs` example
//!
//! The following wiring is assumed:
//! - LED => GPIO1

//% CHIPS: esp32c6
//% FEATURES: esp-hal/unstable

#![no_std]
#![no_main]

use esp_backtrace as _;
use esp_hal::{
    gpio::lp_io::{LowPowerInput, LowPowerOutput},
    load_lp_code,
    lp_core::{LpCore, LpCoreWakeupSource},
    main,
    rtc_cntl::{sleep::WakeFromLpCoreWakeupSource, Rtc},
    uart::{Config, lp_uart::LpUart}
};
use esp_println::{print, println};

#[main]
fn main() -> ! {
    let peripherals = esp_hal::init(esp_hal::Config::default());

    let lp_pin = LowPowerOutput::new(peripherals.GPIO1);

    let uart_cfg = Config::default();
    let tx = LowPowerOutput::new(peripherals.GPIO5);
    let rx = LowPowerInput::new(peripherals.GPIO4);
    let lp_uart = LpUart::new(peripherals.LP_UART, uart_cfg, tx, rx);

    let mut rtc = Rtc::new(peripherals.LPWR);
    let reason = WakeFromLpCoreWakeupSource::default();

    let mut lp_core = LpCore::new(peripherals.LP_CORE);
    lp_core.stop();
    println!("lp-core -> stop");

    // load code to LP core
    let lp_core_code =
        load_lp_code!("./target/riscv32imac-unknown-none-elf/release/lp-core");

    // start LP core
    lp_core_code.run(&mut lp_core, LpCoreWakeupSource::HpCpu, lp_uart, lp_pin);
    println!("lpcore -> run");

    let mut last: u32 = 0;
    let data = (0x5000_2000) as *mut u32;
    loop {
        let read = unsafe {
            data.read_volatile()
        };
        if last != read {
            print!("Current {:x}           \u{000d}", read);
            last = read;
        }
        // FIXME...
        // println!("hp-core -> deep sleep");
        // rtc.sleep_light(&[&reason]);
    }
}
